/**

  The Initial Developer of the Original Code is
  Matthieu  - http://www.programmation-facile.com/
  Portions created by the Initial Developer are Copyright (C) 2013
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

 */



/**
 * Le Framework de jeux JavaScript Phaser
 *
Phaser v2.0.5 - http://phaser.io - @photonstorm - (c) 2014 Photon Storm Ltd. 
https://github.com/photonstorm/phaser
http://examples.phaser.io/
 */



/**
 * Les variables du jeu
 * 
 */

var game; 


var ball; // la balle, le héros du jeu
var arrow; // la flèche
var rotateDirection = 1; // direction de la rotation : 1- dans le sens des aiguilles d'une montre, 2- dans le sens inverse
var power = 0; // la puissance du lancer de la balle
var hudText; // affichage du texte
var charging=false; //  puissance en cours de chargement ?
var degToRad=0.0174532925; // conversion degrés - radians
var score = 0; // le score
var coin; // les bonus à toucher
var deadlyArray = []; // tableau avec tous les ennemis
var gameOver = false; // jeu terminé ?

// configuration du gameplay du jeu
var friction = 0.99; // les frottements sur la balle
var ballRadius = 10; // le rayon des éléments
var rotateSpeed = 3; // vitesse de rotation de la flèche
var minPower = 50; // puissance minimum sur la balle
var maxPower = 200; // puissance maximum sur la balle

var nWidth = 640;
var nHeight = 480;


document.addEventListener("DOMContentLoaded", Main, false); // appel au chargement de la page


/**
 * Fonction principale
 * Appelée au chargement de la page
 * 
 */
function Main()
{
    console.log("Main");

    // creation de la zone de jeu - API canvas
    game = new Phaser.Game(nWidth, nHeight, Phaser.AUTO, "idGameDiv", {preload:onPreload, create:onCreate, update:onUpdate});
}


/**
 * Préchargement des éléments du jeu
 * 
 * @return {[type]} [description]
 */
function onPreload()
{ 
    game.stage.backgroundColor = '#cccccc';// la couleur de fond du 

    game.load.image("ball", "assets/ball.png");// charge les images du jeu
    game.load.image("deadly", "assets/deadly.png");


    var nTypeMap = getRandomInt(1,2);// Génère un nombre entier aléatoire entre min (inclus) et max (inclus)

    // chargement d'un bonus aléatoire
    if( nTypeMap == 1 )
        game.load.image("bonus", "assets/bonus1.jpg");
    else
        game.load.image("bonus", "assets/bonus2.png");

    
    game.load.image("arrow", "assets/arrow.png");
}


/**
 * Génère un nombre entier aléatoire entre min (inclus) et max (inclus)
 * 
 */
function getRandomInt(min, max)
{
    return Math.floor( Math.random() * (max - min + 1) ) + min;
}


/**
 * Création de la scène du jeu,
 * mise en place des graphismes...
 * 
 * @return {[type]} [description]
 */
function onCreate() 
{
    // le jeu en plein écran
    /*
    game.scale.pageAlignHorizontally = true;
    game.scale.pageAlignVertically = true;
    game.scale.scaleMode = Phaser.ScaleManager.SHOW_ALL;
    game.scale.setScreenSize();
    */   
   
    // ajoute la balle au centre du jeu
    ball = game.add.sprite(game.world.centerX,game.world.centerY, "ball");
    ball.anchor.x = 0.5;
    ball.anchor.y = 0.5;
    
    // vitesse de la balle au lancement du jeu
    ball.xSpeed = 0;
    ball.ySpeed = 0;
    
    // la rotation de la flèche
    arrow = game.add.sprite(game.world.centerX,game.world.centerY, "arrow");
    arrow.anchor.x = -1;
    arrow.anchor.y = 0.5;
    
    addEnnemi();// ajoute les ennemis
    
    // création et ajout des bonus
    coin = game.add.sprite(Math.random()*400+40,Math.random()*240+40, "bonus");
    coin.anchor.x = 0.5;
    coin.anchor.y = 0.5;
    addBonus();
    
    // ajout du texte informatif
    hudText = game.add.text(5,0,"",{ 
        font: "20px Arial",
        fill: "#006f77", 
        align: "left" 
    });
    
    updateTexte();// mise à jour du texte informatif
    game.input.onDown.add(charge, this);// écouteur sur le clic de la souris
}


/**
 * Appelée en continu pendant le jeu
 * 
 * @return {[type]} [description]
 */
function onUpdate() 
{
    // mise à jour du jeu uniquement si le joueur est toujours en course ;)
    if(!gameOver)
    {
        // quand le joueur augmente la puissance du lancer
        if(charging)
        {
            power++;
            power = Math.min(power,maxPower);
            updateTexte();        
        } 
        //  sinon continue de faire tourner la flèche du héros
        else
            arrow.angle+=rotateSpeed*rotateDirection;
        
        // mets à jour la position du héros en fonction de la vitesse
        ball.x += ball.xSpeed;
        ball.y += ball.ySpeed;
        
        checkRebondMurs();// gestion du rebond sur les murs
        
        // réduit la vitesse de la balle avec le frottement
        ball.xSpeed *= friction;
        ball.ySpeed *= friction;
        
        // mise à jour de la position de la flèche
        arrow.x = ball.x;
        arrow.y = ball.y;
        
        // si le joueur touche un bonus, alors mise à jour du texte informatif, ajoute un ennemi et un nouveau bonus
        if( getDistance(ball,coin) < (ballRadius*2) * (ballRadius*2) )
        {
            score += 1;
            addEnnemi();
            addBonus();
            updateTexte();    
        }
        
        // si le joueur touche un ennemi --> gameOver
        for(i=0;i<deadlyArray.length;i++)
        {
            if(getDistance(ball,deadlyArray[i])<(ballRadius*2)*(ballRadius*2))
            {
                gameOver = true;
                updateTexte(); 
            }   
        }
    }
}


/**
 * ajoute un ennemi sur le jeu
 * 
 * @return {[type]} [description]
 */
function addEnnemi()
{
    // ajoute le graphisme d'un ennemi
    var deadly = game.add.sprite(0, 0, "deadly");
    deadly.anchor.x = 0.5;
    deadly.anchor.y = 0.5;
    
    deadlyArray.push(deadly);// ajoute l'ennemi dans le tableau contenant tous les ennemis
    
    // place l'ennemi à une position aléatoire sur le jeu
    do{
        var randomX = Math.random() * (game.width-2*ballRadius) + ballRadius;
        var randomY = Math.random() * (game.height-2*ballRadius) + ballRadius;

        deadlyArray[deadlyArray.length-1].x = randomX;
        deadlyArray[deadlyArray.length-1].y = randomY;

    } while (checkPosEnnemi());
}


/**
 * Est ce que la position de l'ennemi est correct 
 * (sur la carte du jeu)
 * 
 * @return {[type]} [description]
 */
function checkPosEnnemi()
{   
    // si la distance entre l'ennemi et le héros est trop petite
    if( getDistance(ball,deadlyArray[deadlyArray.length-1]) < (ballRadius*3)*(ballRadius*3) )
        return true;
    
    // si la distance entre le nouveau ennemi est les autres est trop petite
    for(i=0;i<deadlyArray.length-1;i++)
    {
        if( getDistance(deadlyArray[i],deadlyArray[deadlyArray.length-1]) < (ballRadius*2)*(ballRadius*2) )
            return true;
    }
    
    // position correcte
    return false;
}


/**
 * Ajoute un bonus sur le jeu
 * 
 * @return {[type]} [description]
 */
function addBonus()
{
    // place le bonus à une position aléatoire sur le jeu
    do{
        coin.x = Math.random()*(game.width-2*ballRadius)+ballRadius;
        coin.y = Math.random()*(game.height-2*ballRadius)+ballRadius;
    } while (checkPosBonus());
}


/**
 * Est ce que la position du Bonus est correct 
 * (sur la carte du jeu)
 * 
 * @return {[type]} [description]
 */
function checkPosBonus()
{
    // si la position entre le bonus et le héros est trop petite
    if( getDistance(ball,coin) < (ballRadius*2.5) * (ballRadius*2.5) )
        return true;
    
    // si la position entre le bonus et les ennemis est trop petite
    for(i=0;i<deadlyArray.length;i++)
    {
        if( getDistance(deadlyArray[i],coin) < (ballRadius*3)*(ballRadius*3) )
            return true;
    }
    
    // position correcte
    return false;   
}


/**
 * Gestion du rebond du joueur avec les rebords du jeu
 * 
 * @return {[type]} [description]
 */
function checkRebondMurs()
{
    if(ball.x < ballRadius)
    {
        ball.x = ballRadius;
        ball.xSpeed *= -1
    }

    if(ball.y < ballRadius)
    {
        ball.y = ballRadius;
        ball.ySpeed *= -1;
    }

    if(ball.x > game.width-ballRadius)
    {
        ball.x=game.width-ballRadius;
        ball.xSpeed *= -1;
    }

    if(ball.y > game.height-ballRadius)
    {
        ball.y=game.height-ballRadius;
        ball.ySpeed *= -1;
    }    
}


/**
 * Récupère la distance entre 2 sprites
 * Pas d'utilisation de sqrt pour économiser des ressources CPU
 * 
 * @param  {[type]} from [description]
 * @param  {[type]} to   [description]
 * @return {[type]}      [description]
 */
function getDistance(from, to)
{
    var xDist = from.x - to.x
    var yDist = from.y - to.y;

    return xDist*xDist+yDist*yDist;
}


/**
 * Chargement de la puissance du lancer.
 * 
 * @return {[type]} [description]
 */
function charge()
{
    power = minPower;

    game.input.onDown.remove(charge, this); // supprime l'écouteur sur la souris (clic)
    game.input.onUp.add(fire, this);  // ajoute un écouteur sur le relachement du clic de la souris

    charging=true;// chargement en cours de la puissance du lancer
}


/**
 * Déclenchement du tir,
 * déplacement du héros
 * inverse la rotation de la flèche
 * mise à jiour du texte informatif
 * 
 * @return {[type]} [description]
 */
function fire()
{
    // mise à jour écouteur de la souris
    game.input.onUp.remove(fire, this); 
    game.input.onDown.add(charge, this);

    ball.xSpeed += Math.cos(arrow.angle*degToRad) * power/20;
    ball.ySpeed += Math.sin(arrow.angle*degToRad) * power/20;

    power = 0;
    updateTexte();

    charging = false; 
    rotateDirection *= -1;
}


/**
 * Texte informatif du jeu
 * 
 * @return {[type]} [description]
 */
function updateTexte()
{
    if( !gameOver )
        hudText.text = "Puissance : "+power+" - Score : "+score; 
    else
        hudText.text = "GAME OVER !!! --> Votre Score : "+score;   
}



